@@ -28,6 +28,10 @@ static const R_CallMethodDef callMethods[] = {
 	{"R_zmq_recv_char", (DL_FUNC) &R_zmq_recv_char, 3},
 	{"R_zmq_recv_raw", (DL_FUNC) &R_zmq_recv_raw, 3},
 
+	/* In file "R_zmq_transfer.c". */
+	{"R_zmq_send_file", (DL_FUNC) &R_zmq_send_file, 7},
+	{"R_zmq_recv_file", (DL_FUNC) &R_zmq_recv_file, 7},
+
 	/* In file "R_zmq_utility.c". */
 	{"R_zmq_strerror", (DL_FUNC) &R_zmq_strerror, 1},
 	{"R_zmq_version", (DL_FUNC) &R_zmq_version, 0},
@@ -36,12 +40,10 @@ static const R_CallMethodDef callMethods[] = {
 	{"shellexec_wcc", (DL_FUNC) &shellexec_wcc, 2},
 
 	/* In file "R_zmq_poll.c". */
-	{"R_zmq_poll", (DL_FUNC) &R_zmq_poll, 3},
+	{"R_zmq_poll", (DL_FUNC) &R_zmq_poll, 4},
 	{"R_zmq_poll_free", (DL_FUNC) &R_zmq_poll_free, 0},
 	{"R_zmq_poll_length", (DL_FUNC) &R_zmq_poll_length, 0},
 	{"R_zmq_poll_get_revents", (DL_FUNC) &R_zmq_poll_get_revents, 1},
-	{"R_zmq_poll2", (DL_FUNC) &R_zmq_poll2, 3},
-	{"R_zmq_poll2_get_revents", (DL_FUNC) &R_zmq_poll2_get_revents, 2},
 
 	/* Finish R_CallMethodDef. */
 	{NULL, NULL, 0}
@@ -51,4 +53,5 @@ static const R_CallMethodDef callMethods[] = {
 
 void R_init_pbdZMQ(DllInfo *info){
 	R_registerRoutines(info, NULL, callMethods, NULL, NULL);
+	R_useDynamicSymbols(info, FALSE);
 } /* End of R_init_pbdZMQ(). */
